import { __awaiter, __decorate } from "tslib";
import { customElement, attr, observable } from '@microsoft/fast-element';
import { SystemHealthAlertsBase } from '../foundation-system-health-alerts-base';
import { logger } from '../utils';
import { SYSTEM_HEALTH_ALERTS_SERVICE_LOGGER_PREFIX as LOGGER_PREFIX, SYSTEM_HEALTH_ALERTS_SERVICE_EVENTS_NAMES as EVENTS_NAMES, SYSTEM_HEALTH_ALERTS_SERVICE_CLOSE_TIMEOUT as CLOSE_TIMEOUT, } from './main.config';
import { foundationSystemHealthAlertsServiceStyles as styles } from './main.styles';
import { getPrefixedSystemHealthAlertsService as getTemplate } from './main.template';
/**
 * The `SystemHealthAlertsService` component is designed to monitor and display the status of various services in real-time.
 * It leverages a connection to a backend datasource to receive updates about services status
 * and displays these updates to the user.
 *
 * @alpha
 */
export class SystemHealthAlertsService extends SystemHealthAlertsBase {
    constructor() {
        super(...arguments);
        this.closeTimeout = CLOSE_TIMEOUT;
        this.autoClose = false;
        this.maxMessages = 1;
        this.messageRestored = `Thank you for your patience while we were restoring service. The [SERVICE_NAME] service is now available as normal. Please refresh your page to update`;
        this.messageFailed = `We're sorry, [SERVICE_NAME] service is experiencing technical difficulties. It is recommended that you save your data before disconnecting from the application. Data is not current and may be incomplete`;
        this.titleRestored = `[SERVICE_NAME] Service back online`;
        this.titleFailed = `[SERVICE_NAME] Service outage`;
        this.allMessages = [];
        this.displayMessages = [];
        this.systemStatusServices = [];
        this.closeMessageHandler = (event) => {
            const serviceTag = event.detail;
            this.allMessages = this.allMessages.filter((message) => message.tag !== serviceTag);
        };
        this.getTextHelper = (textType, type, item) => {
            const getterByType = {
                title: {
                    restored: this.titleRestored,
                    failed: this.titleFailed,
                },
                message: {
                    restored: this.messageRestored,
                    failed: this.messageFailed,
                },
            };
            const message = getterByType[textType][type];
            return this.replaceTextPlaceholders(message, item);
        };
    }
    maxMessagesChanged() {
        this.updateDisplayMessages();
    }
    allMessagesChanged(ignore, currentMessages) {
        if (currentMessages === null || currentMessages === void 0 ? void 0 : currentMessages.length) {
            this.$emit(EVENTS_NAMES.CHANGED, currentMessages);
            logger.debug(`${LOGGER_PREFIX}new messages`, currentMessages);
            this.updateDisplayMessages();
        }
    }
    systemStatusServicesChanged(oldServices, newServices) {
        if (oldServices === null || oldServices === void 0 ? void 0 : oldServices.length) {
            const changedServices = newServices.filter((newService) => {
                const oldService = oldServices.find(({ tag }) => tag === newService.tag);
                return oldService && oldService.isConnected !== newService.isConnected;
            });
            if (changedServices.length) {
                this.allMessages = [
                    ...this.allMessages.filter(({ tag }) => !changedServices.some(({ tag: newTag }) => tag === newTag)),
                    ...changedServices,
                ];
                changedServices.forEach((service) => {
                    if (service.isConnected) {
                        this.$emit(EVENTS_NAMES.RESTORED, service);
                    }
                    else {
                        this.$emit(EVENTS_NAMES.FAIL, service);
                    }
                });
            }
        }
    }
    systemStatusChanged(ignore, newStatus) {
        const { resources: services } = newStatus;
        const currentServices = [];
        Object.keys(services).forEach((serviceTag) => {
            const service = services[serviceTag];
            const serviceItem = this.prepareServiceItem(service, serviceTag);
            currentServices.push(serviceItem);
        });
        this.systemStatusServices = currentServices;
    }
    updateDisplayMessages() {
        var _a;
        this.displayMessages =
            this.maxMessages && ((_a = this.allMessages) === null || _a === void 0 ? void 0 : _a.length)
                ? this.allMessages.slice(-this.maxMessages)
                : this.allMessages;
    }
    prepareServiceItem(systemStatusItem, tag) {
        return {
            tag,
            isConnected: systemStatusItem.isConnected,
            name: systemStatusItem.name,
        };
    }
    emitCloseMessage(target, identifier) {
        target.dispatchEvent(new CustomEvent(EVENTS_NAMES.CLOSE_MESSAGE, {
            bubbles: true,
            composed: true,
            detail: identifier,
        }));
    }
    replaceTextPlaceholders(template, item) {
        var _a, _b, _c;
        return template
            .replace(/\[SERVICE_NAME\]/g, (_a = item.name) !== null && _a !== void 0 ? _a : 'Unknown Name')
            .replace(/\[SERVICE_ID\]/g, (_c = (_b = item.tag) === null || _b === void 0 ? void 0 : _b.toString()) !== null && _c !== void 0 ? _c : 'Unknown tag');
    }
}
__decorate([
    attr({ attribute: 'close-timeout' })
], SystemHealthAlertsService.prototype, "closeTimeout", void 0);
__decorate([
    attr({ attribute: 'auto-close', mode: 'boolean' })
], SystemHealthAlertsService.prototype, "autoClose", void 0);
__decorate([
    attr({ attribute: 'max-messages' })
], SystemHealthAlertsService.prototype, "maxMessages", void 0);
__decorate([
    attr({ attribute: 'message-restored' })
], SystemHealthAlertsService.prototype, "messageRestored", void 0);
__decorate([
    attr({ attribute: 'message-failed' })
], SystemHealthAlertsService.prototype, "messageFailed", void 0);
__decorate([
    attr({ attribute: 'title-restored' })
], SystemHealthAlertsService.prototype, "titleRestored", void 0);
__decorate([
    attr({ attribute: 'title-failed' })
], SystemHealthAlertsService.prototype, "titleFailed", void 0);
__decorate([
    observable
], SystemHealthAlertsService.prototype, "allMessages", void 0);
__decorate([
    observable
], SystemHealthAlertsService.prototype, "displayMessages", void 0);
__decorate([
    observable
], SystemHealthAlertsService.prototype, "systemStatusServices", void 0);
/**
 * ZeroSystemHealthAlertsService.
 *
 * @remarks
 * A zero version that pre-registers zero components and uses the zero design system.
 *
 * @example
 * ```ts
 * import { ZeroSystemHealthAlertsService } from '@genesislcap/foundation-system-health-alerts';
 * ...
 * ZeroSystemHealthAlertsService
 * ```
 *
 * @example Load the micro frontend on-demand
 * ```ts

 *   const { ZeroSystemHealthAlertsService } = await import('@genesislcap/foundation-system-health-alerts');
 *   return ZeroSystemHealthAlertsService;
 * },
 * ```
 *
 * @public
 */
let ZeroSystemHealthAlertsService = class ZeroSystemHealthAlertsService extends SystemHealthAlertsService {
    /**
     * @internal
     */
    connectedCallback() {
        const _super = Object.create(null, {
            connectedCallback: { get: () => super.connectedCallback }
        });
        return __awaiter(this, void 0, void 0, function* () {
            _super.connectedCallback.call(this);
            yield this.loadRemotes();
        });
    }
    /**
     * @internal
     */
    loadRemotes() {
        return __awaiter(this, void 0, void 0, function* () {
            const { registerCommonZeroComponents } = yield import('../components/zero-components');
            yield registerCommonZeroComponents();
        });
    }
};
ZeroSystemHealthAlertsService = __decorate([
    customElement({
        name: 'zero-system-health-alerts-service',
        template: getTemplate('zero'),
        styles,
    })
], ZeroSystemHealthAlertsService);
export { ZeroSystemHealthAlertsService };
